// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       message_holder.h
 *  \brief      メッセージホルダー
 *  \date       Since: June 29, 2021. 16:49:28 JST.
 *  \author     Acerola
 *  \version    1.0.2
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MESSAGE_HOLDER_H_1624952968
#define INCGUARD_MESSAGE_HOLDER_H_1624952968

#include <mgl/mgl.h>

namespace MGLExt
{
//! メッセージホルダークラス
class MessageHolder : public MGL::StaticSingleton<MessageHolder>
{
public:
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     */
    /* ------------------------------------------------------------------------- */
    MessageHolder() noexcept
        : _hashSeed(MGL::Hash::kFNV1aDefaultValue32)
        , _labelCount(0)
        , _languageCount(0)
        , _isIndexed(false)
        , _labelTable(nullptr)
        , _languageTable(nullptr)
        , _offsetPool(nullptr)
        , _messagePool(nullptr)
        , _currentLanguage(0)
        , _eventChangeLanguage(MGL::Event::NotifyType::ChangeLanguage, OnEventChangeLanguage, this)
    {
    }

    bool Load(const char *path) noexcept;

    bool SetLanguage(uint32_t languageHash) noexcept;
    bool SetLanguage(MGL::System::Language language) noexcept;

    size_t GetIndex(uint32_t labelHash) const noexcept;

    const char *GetMessage(size_t index) noexcept;
    const MGL::Text::IndexedCharacter *GetIndexedMessage(size_t index) noexcept;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      このホルダーが保持しているラベルの数を取得
     *  \return     ラベルの数
     */
    /* ------------------------------------------------------------------------- */
    constexpr size_t GetLabelCount() const noexcept
    {
        return static_cast<size_t>(_labelCount);
    }

    static void OnEventChangeLanguage(void *callbackArg, void *notifyArg) noexcept;

private:
    uint32_t _hashSeed;
    uint32_t _labelCount;
    uint32_t _languageCount;
    bool _isIndexed;
    MGL::STL::unique_ptr<uint32_t[]> _labelTable;
    MGL::STL::unique_ptr<uint32_t[]> _languageTable;
    MGL::STL::unique_ptr<uint32_t[]> _offsetPool;
    MGL::STL::unique_ptr<std::byte[]> _messagePool;

    size_t _currentLanguage;
    
    MGL::Event::Handle _eventChangeLanguage;
};
}   // namespace MGLExt

#endif	// INCGUARD_MESSAGE_HOLDER_H_1624952968

// vim: et ts=4 sw=4 sts=4
